@@ -1,7 +1,7 @@
 /*
  * This file is part of Bino, a 3D video player.
  *
- * Copyright (C) 2022, 2023
+ * Copyright (C) 2022, 2023, 2024
  * Martin Lambers <marlam@marlam.de>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,12 +28,6 @@
 #include "tools.hpp"
 #include "metadata.hpp"
 
-/* These might not be defined in OpenGL ES environments.
- * Define them here to fix compilation. */
-#ifndef GL_TEXTURE_MAX_ANISOTROPY_EXT
-# define GL_TEXTURE_MAX_ANISOTROPY_EXT 0x84FE
-#endif
-
 
 static Bino* binoSingleton = nullptr;
 
@@ -44,11 +38,14 @@ Bino::Bino(const Screen& screen, bool swapEyes) :
     _player(nullptr),
     _audioInput(nullptr),
     _videoInput(nullptr),
+    _screenInput(nullptr),
+    _windowInput(nullptr),
     _captureSession(nullptr),
     _lastFrameInputMode(Input_Unknown),
     _lastFrameSurroundMode(Surround_Unknown),
     _screen(screen),
     _frameIsNew(false),
+    _frameWasSerialized(true),
     _swapEyes(swapEyes)
 {
     Q_ASSERT(!binoSingleton);
@@ -62,6 +59,8 @@ Bino::~Bino()
     delete _player;
     delete _audioInput;
     delete _videoInput;
+    delete _screenInput;
+    delete _windowInput;
     delete _captureSession;
     binoSingleton = nullptr;
 }
@@ -75,6 +74,7 @@ void Bino::initializeOutput(const QAudioDevice& audioOutputDevice)
 {
     _videoSink = new VideoSink(&_frame, &_extFrame, &_frameIsNew);
     connect(_videoSink, &VideoSink::newVideoFrame, [=]() { emit newVideoFrame(); });
+    connect(_videoSink, &VideoSink::newVideoFrame, [=]() { _frameWasSerialized = false; });
     _audioOutput = new QAudioOutput;
     _audioOutput->setDevice(audioOutputDevice);
 }
@@ -120,29 +120,59 @@ void Bino::stopPlaylistMode()
     }
 }
 
-void Bino::startCaptureMode(
-        bool withAudioInput,
-        const QAudioDevice& audioInputDevice,
-        const QCameraDevice& videoInputDevice)
+void Bino::startCaptureMode(bool withAudioInput, const QAudioDevice& audioInputDevice)
 {
     if (playlistMode())
         stopPlaylistMode();
     if (captureMode())
         stopCaptureMode();
 
+    _captureSession = new QMediaCaptureSession;
+    _captureSession->setAudioOutput(_audioOutput);
+    _captureSession->setVideoSink(_videoSink);
     if (withAudioInput) {
         _audioInput = new QAudioInput;
         _audioInput->setDevice(audioInputDevice);
+        _captureSession->setAudioInput(_audioInput);
     }
+}
+
+void Bino::startCaptureModeCamera(
+        bool withAudioInput,
+        const QAudioDevice& audioInputDevice,
+        const QCameraDevice& videoInputDevice)
+{
+    startCaptureMode(withAudioInput, audioInputDevice);
     _videoInput = new QCamera;
     _videoInput->setCameraDevice(videoInputDevice);
-    _captureSession = new QMediaCaptureSession;
-    if (_audioInput)
-        _captureSession->setAudioInput(_audioInput);
     _captureSession->setCamera(_videoInput);
-    _captureSession->setAudioOutput(_audioOutput);
-    _captureSession->setVideoSink(_videoSink);
+    _videoInput->setActive(true);
+    emit stateChanged();
+}
 
+void Bino::startCaptureModeScreen(
+        bool withAudioInput,
+        const QAudioDevice& audioInputDevice,
+        QScreen* screenInputDevice)
+{
+    startCaptureMode(withAudioInput, audioInputDevice);
+    _screenInput = new QScreenCapture;
+    _screenInput->setScreen(screenInputDevice);
+    _captureSession->setScreenCapture(_screenInput);
+    _screenInput->setActive(true);
+    emit stateChanged();
+}
+
+void Bino::startCaptureModeWindow(
+        bool withAudioInput,
+        const QAudioDevice& audioInputDevice,
+        const QCapturableWindow& windowInputDevice)
+{
+    startCaptureMode(withAudioInput, audioInputDevice);
+    _windowInput = new QWindowCapture;
+    _windowInput->setWindow(windowInputDevice);
+    _captureSession->setWindowCapture(_windowInput);
+    _windowInput->setActive(true);
     emit stateChanged();
 }
 
@@ -153,6 +183,10 @@ void Bino::stopCaptureMode()
         _captureSession = nullptr;
         delete _videoInput;
         _videoInput = nullptr;
+        delete _screenInput;
+        _screenInput = nullptr;
+        delete _windowInput;
+        _windowInput = nullptr;
         if (_audioInput) {
             delete _audioInput;
             _audioInput = nullptr;
@@ -473,28 +507,31 @@ void Bino::deserializeStaticData(QDataStream& ds)
     ds >> _screen;
 }
 
-void Bino::serializeDynamicData(QDataStream& ds) const
+void Bino::serializeDynamicData(QDataStream& ds)
 {
-    ds << _frameIsNew;
-    if (_frameIsNew) {
+    ds << _frameWasSerialized;
+    if (!_frameWasSerialized) {
         ds << _frame;
         if (_frame.inputMode == Input_Alternating_LR
                 || _frame.inputMode == Input_Alternating_RL) {
             ds << _extFrame;
         }
+        _frameWasSerialized = true;
     }
     ds << _swapEyes;
 }
 
 void Bino::deserializeDynamicData(QDataStream& ds)
 {
-    ds >> _frameIsNew;
-    if (_frameIsNew) {
+    bool noNewFrame;
+    ds >> noNewFrame;
+    if (!noNewFrame) {
         ds >> _frame;
         if (_frame.inputMode == Input_Alternating_LR
                 || _frame.inputMode == Input_Alternating_RL) {
             ds >> _extFrame;
         }
+        _frameIsNew = true;
     }
     ds >> _swapEyes;
 }
@@ -507,6 +544,7 @@ bool Bino::wantExit() const
 bool Bino::initProcess()
 {
     bool isGLES = QOpenGLContext::currentContext()->isOpenGLES();
+    bool haveAnisotropicFiltering = checkTextureAnisotropicFilterAvailability();
     LOG_DEBUG("Using OpenGL in the %s variant", isGLES ? "ES" : "Desktop");
 
     // Qt-based OpenGL initialization
@@ -667,14 +705,16 @@ bool Bino::initProcess()
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 4.0f);
+    if (haveAnisotropicFiltering)
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY, 4.0f);
     glGenTextures(1, &_extFrameTex);
     glBindTexture(GL_TEXTURE_2D, _extFrameTex);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 4.0f);
+    if (haveAnisotropicFiltering)
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY, 4.0f);
     CHECK_GL();
 
     // Subtitle texture
@@ -684,7 +724,8 @@ bool Bino::initProcess()
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
-    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 4.0f);
+    if (haveAnisotropicFiltering)
+        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY, 4.0f);
     CHECK_GL();
 
     // Screen geometry
